# Copyright 2009-2022 Joshua Bronson. All rights reserved.
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""Define bidict package metadata."""


__version__ = '0.22.0'
__author__ = 'Joshua Bronson'
__maintainer__ = 'Joshua Bronson'
__copyright__ = 'Copyright 2009-2022 Joshua Bronson'
__email__ = 'jabronson@gmail.com'
__description__ = 'The bidirectional mapping library for Python.'
__keywords__ = 'dict dictionary mapping datastructure bimap bijection bijective ' \
    'injective inverse reverse bidirectional two-way 2-way'
__license__ = 'MPL 2.0'
__status__ = 'Beta'
__url__ = 'https://bidict.readthedocs.io'
__project_urls__ = {
    'Donate': 'https://github.com/sponsors/jab',
    'Documentation': 'https://bidict.readthedocs.io',
    'Enterprise Support': 'https://bidict.readthedocs.io/#enterprise-support',
    'Changelog': 'https://bidict.readthedocs.io/changelog.html',
    'Source Code': 'https://github.com/jab/bidict',
    'Issue Tracker': 'https://github.com/jab/bidict/issues',
    'Chat': 'https://gitter.im/jab/bidict',
}
